/***************************************************************************

	Callback-DLL zum Protokollieren der Meldungen aus der EURO-FIBU
	zur Laufzeit

	ERSTELLT : Januar '96 von th

	LETZTE ERGNZUNG : am 25.01.96 von th

	FUNKTIONEN:
		DllMain()				-	Hauptroutine der DLL
		OFCK_OUT()				-	Callback-Funktion, die von der EURO-FIBU
										aufgerufen wird

**************************************************************************/

#include <windows.h>
#include <ofck.h>

HINSTANCE ghinstDLL;
HANDLE ghOutputFile;
BOOL gbError;

static char sszOutputFileName[] = "LOG_OFCK.TXT";
static char sszConstNoName[] = "undefinierter Wert";

// Hilfsstruktur fr Konstantenbenennung
typedef struct
{
	LONG lValue;			// Konstantenwert
	LPSTR lpszName;		// Name der Konstanten
} CONSTNAME, *LPCONSTNAME;

// Konstantenstrukturen
CONSTNAME gcnArt[] = {
	{OFCK_SAKO, "OFCK_SAKO"}, {OFCK_DEBI, "OFCK_DEBI"},
	{OFCK_KRED, "OFCK_KRED"}, {OFCK_BUBE, "OFCK_BUBE"}
};
CONSTNAME gcnAktion[] = {
	{OFCK_UPDATE, "OFCK_UPDATE"}, {OFCK_DELETE, "OFCK_DELETE"},
	{OFCK_INSERT, "OFCK_INSERT"}
};
CONSTNAME gcnKlasse[] = {
	{OFCK_OHNE, "OFCK_OHNE"}, {OFCK_BILAKTIV, "OFCK_BILAKTIV"},
	{OFCK_BILPASSIV, "OFCK_BILPASSIV"}, {OFCK_GUNDV, "OFCK_GUNDV"}
};
CONSTNAME gcnKostenkennung[] = {
	{OFCK_FIX, "OFCK_FIX"}, {OFCK_VARIABEL, "OFCK_VARIABLE"}
};

// Konstanten fr Datenausgabe
#define TYPE_STRING 0
#define TYPE_LONG	1
#define TYPE_SEPARATOR 2



// Funktionen fr OFCK_OUT
void FileOutput(int iType, LPVOID lpData = NULL, LPSTR lpszTitle = NULL);
LPSTR GetConstantName(LONG lValue, LPCONSTNAME lpcnFirst, short iNum);

// Definition fr Funktionsexport
#define DLLEXPORT __declspec(dllexport)

////////////////////////////////////////////////////////////////////////////
// OFCK_OUT
////////////////////////////////////////////////////////////////////////////
// Callback-Funktion fr die EURO-FIBU. Bei Vernderung des Datenbestandes
// wird diese Funktion aufgerufen.
// iAktion		Aktionstyp e {OFCK_INSERT, OFCK_DELETE, OFCK_UPDATE}
// iArt			Datenart e {OFCK_KRED, OFCK_DEBI, OFCK_SAKO, OFCK_BUBE}
// lpData		vernderter Datensatz
void OFCK_OUT(int iAktion, int iArt, LPVOID lpData)
{
	// Gib zunchst Datenart und Aktion aus
	FileOutput(TYPE_STRING, GetConstantName(iArt, gcnArt, 4), "Art der Daten");
	FileOutput(TYPE_STRING, GetConstantName(iAktion, gcnAktion, 3), "Aktion");
	FileOutput(TYPE_STRING, "");
	switch (iArt)
	{
		case OFCK_SAKO:
			{
				LPOFCKSK	lpsk;

				// Vernderung eines Sachkontos, die Daten sind in Form
				// einer OFCK_SACHKONTO-Struktur. Caste deshalb
				lpsk = (LPOFCKSK)lpData;
				FileOutput(TYPE_LONG, (LPVOID)lpsk->mandnr, "Mandant");
				FileOutput(TYPE_STRING, lpsk->kontonr, "Kontonummer");
				FileOutput(TYPE_STRING, lpsk->kuerzel, "Krzel");
				FileOutput(TYPE_STRING, lpsk->name, "Name");
				FileOutput(TYPE_STRING, GetConstantName(lpsk->klasse,
					gcnKlasse, 4), "Klasse");
				FileOutput(TYPE_LONG, (LPVOID)lpsk->isumsatzerloes, "Umsatzerlskonto");
				FileOutput(TYPE_LONG, (LPVOID)lpsk->iskostenart, "Kostenart");
			}
			break;
		case OFCK_KRED:
		case OFCK_DEBI:
			{
				LPOFCKPK lppk;

				// Vernderung eines Personenkontos, die Daten sind in Form
				// einer OFCK_PEKO-Struktur. Caste deshalb
				lppk = (LPOFCKPK)lpData;
				FileOutput(TYPE_LONG, (LPVOID)lppk->mandnr, "Mandant");
				FileOutput(TYPE_STRING, lppk->kontonr, "Kontonummer");
				FileOutput(TYPE_STRING, lppk->kuerzel, "Krzel");
				FileOutput(TYPE_STRING, lppk->name, "Name");
				FileOutput(TYPE_STRING, lppk->zusatz, "Zusatz");
				FileOutput(TYPE_STRING, lppk->ustidnr, "UStIDNr.");
				FileOutput(TYPE_STRING, lppk->land, "Land");
				FileOutput(TYPE_STRING, lppk->waehrung, "Whrung");
				FileOutput(TYPE_STRING, lppk->plz, "PLZ");
				FileOutput(TYPE_STRING, lppk->ort, "Ort");
				FileOutput(TYPE_STRING, lppk->strasse, "Strae");
				FileOutput(TYPE_STRING, lppk->blz, "BLZ");
				FileOutput(TYPE_STRING, lppk->bankknr, "Bankktonr.");
				FileOutput(TYPE_STRING, lppk->bank, "Bank");
				FileOutput(TYPE_STRING, lppk->gegenkonto, "Gegenkonto");
				FileOutput(TYPE_LONG, (LPVOID)lppk->skontotage, "Skontotage");
				FileOutput(TYPE_LONG, (LPVOID)lppk->zahltage, "Zahltage");
				FileOutput(TYPE_STRING, lppk->name, "Name");
				FileOutput(TYPE_STRING, lppk->skontoproz, "Skontoprozent");
				FileOutput(TYPE_LONG, (LPVOID)lppk->mahnkennziffer, "Mahnkennziffer");
			}
			break;
		case OFCK_BUBE:
			{
				LPOFCKSPLITBUC lpsb;
				LPOFCKTEILBUC lptb;
				short iCount;

				// Es wurde gebucht. Die Daten sind in Form einer
				// einer OFCK_SPLITBUCHUNG-Struktur. Caste deshalb
				lpsb = (LPOFCKSPLITBUC)lpData;
				// Gib zunchst die "globalen" Buchungsdaten aus
				FileOutput(TYPE_LONG, (LPVOID)lpsb->mandnr, "Mandant");
				FileOutput(TYPE_LONG, (LPVOID)lpsb->stapelnr, "Stapelnummer");
				FileOutput(TYPE_STRING, lpsb->belegdatum, "Belegdatum");
				FileOutput(TYPE_STRING, lpsb->belegnr, "Belegnummer");
				FileOutput(TYPE_STRING, lpsb->text, "Text");
				FileOutput(TYPE_LONG, (LPVOID)lpsb->teilbuchungsanz, "Teilbuchungsanzahl");
				FileOutput(TYPE_LONG, (LPVOID)lpsb->ustdirekt, "Umsatzsteuer direkt");
				// Jetzt kommen die einzelnen Teilbuchungen zur Ausgabe
				for (lptb = lpsb->teilbuchung, iCount = 1; 
					iCount <= lpsb->teilbuchungsanz; lptb++, iCount++)
				{
					FileOutput(TYPE_STRING, "");
					FileOutput(TYPE_LONG, (LPVOID)iCount, "Teilbuchungsnummer");
					FileOutput(TYPE_STRING, lptb->sollkonto, "Sollkonto");
					FileOutput(TYPE_STRING, GetConstantName(lptb->sollart, 
						gcnArt, 3), "Sollart");
					FileOutput(TYPE_STRING, lptb->habenkonto, "Habenkonto");
					FileOutput(TYPE_STRING, GetConstantName(lptb->habenart, 
						gcnArt, 3), "Habenart");
					FileOutput(TYPE_STRING, lptb->bruttobetrag, "Bruttobetrag");
					FileOutput(TYPE_STRING, lptb->steusa, "Steuersatz");
					FileOutput(TYPE_STRING, lptb->steuerbetrag, "Steuerbetrag");
					FileOutput(TYPE_STRING, lptb->kostenstelle1, "Kostenstelle 1");
					FileOutput(TYPE_STRING, lptb->kostenstelle2, "Kostenstelle 2");
					FileOutput(TYPE_STRING, GetConstantName(lptb->kostenkennung,
						gcnKostenkennung, 2), "Kostenkennung");
				}
			}		
			break;
	}
	// Trenner zwischen Datenstzen
	FileOutput(TYPE_SEPARATOR, NULL);
}

////////////////////////////////////////////////////////////////////////////
// FileOutput
////////////////////////////////////////////////////////////////////////////
// formatierte Ausgabe in die Protokolldatei.
// iType			Datentyp der Ausgabe (siehe TYPE_-Konstanten)
// lpData		Eigentliche Ausgabedaten
// lpszTitle	Titel, der vor die Ausgabedaten mit einem '=' gestellt wird.
void FileOutput(int iType, LPVOID lpData, LPSTR lpszTitle)
{
	DWORD dwBytes;
	char szOut[300];

	// Falls Fehlerflag gesetzt: nicht schreiben
	if (gbError || (ghOutputFile == INVALID_HANDLE_VALUE))
		return;
	// Eventuell vorangestellter Titel
	if (lpszTitle)
	{
		strcpy(szOut, lpszTitle);
		strcat(szOut, " = ");
	}
	else
		szOut[0] = 0;
	switch (iType)
	{
		case TYPE_STRING:
			strcat(szOut, (LPSTR)lpData);
			break;
		case TYPE_LONG:
			_ltoa((LONG)lpData, szOut + strlen(szOut), 10);
			break;
		case TYPE_SEPARATOR:
			strcpy(szOut, "=================");
			break;
	}
	// Versuche, die Daten zu schreiben
	strcat(szOut, "\n");
	gbError = !WriteFile(ghOutputFile, szOut, strlen(szOut), &dwBytes, NULL);
}

////////////////////////////////////////////////////////////////////////////
// GetConstantName
////////////////////////////////////////////////////////////////////////////
// Sucht in einem Konstanten-Namenfeld nach dem Konstantennamen zum
// gegebenen Wert.
// lValue		Konstanten-Wert
// lpcnFirst	Zeiger auf erste Konstanten-Namensstruktur
// iNum			Anzahl der Konstanten-Namen
// Rckgabe:
//		Zeiger auf Name der Konstanten
LPSTR	GetConstantName(LONG lValue, LPCONSTNAME lpcnFirst, short iNum)
{
	for ( ; iNum; iNum--, lpcnFirst++)
		if (lpcnFirst->lValue == lValue)
			break;
	if (iNum)
		return lpcnFirst->lpszName;
	else
		return sszConstNoName;
}

////////////////////////////////////////////////////////////////////////////
// DllMain
////////////////////////////////////////////////////////////////////////////
// DllEntryPoint-Funktion 
extern "C" BOOL WINAPI DllMain(HINSTANCE hModule, DWORD fdwReason, LPVOID lpReserved) 
{  
	switch (fdwReason)
	{
		case DLL_PROCESS_ATTACH:
			{
				short i;
				char szPfad[MAX_PATH];
				LPSTR lpszScan;

				// ffne die Ausgabedatei zum Schreiben und zwar im
				// DLL-Verzeichnis
				GetModuleFileName(hModule, szPfad, sizeof(szPfad));
				// Scanne den String nach dem letzten Backslash oder Doppelpunkt
				lpszScan = szPfad + strlen(szPfad) - 1;
				for ( ; i; lpszScan--, i--)
					if ((*lpszScan == '\\') || (*lpszScan == ':'))
						break;
				if (i)
					strcpy(lpszScan + 1, sszOutputFileName);
				else
					strcpy(szPfad, sszOutputFileName);
				// Versuche, die Datei zu erzeugen
				ghOutputFile = CreateFile(szPfad, GENERIC_WRITE, 0, NULL,
					CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
				// Setze globales Fehlerflag auf FALSE
				gbError = FALSE;
			}
			break;
		case DLL_PROCESS_DETACH:
			{
				// Schliee die Datei
				if (ghOutputFile != INVALID_HANDLE_VALUE)
					CloseHandle(ghOutputFile);
			}
			break;
	}
	return TRUE;
}

